#include <plib.h>
#include "MyCommands.h"
#include "MyData.h"
#include "NU32.h"
#include "MyLibrary.h"

// This function gets called for the "top-level" command exec.  This
// function takes action depending on what type of sublevel command
// was sent.
void ExecCommand(char *cmd) {
  int i, actual, len, newmode;
  if (ISCMD(cmd, ITUNE)) {
    NU32_WriteUART1("Executing current tuning.\r\n");
    current.desired = CURRENT_AMPLITUDE;
    // save an x number of periods
    len = 6 * TOGGLE_FREQ;
    newmode = TUNE_CURRENT;
  } else if (ISCMD(cmd, HOLD)) {
    // store hold position data
    NU32_WriteUART1("Holding position.\r\n");
    actual = encoder.actual;
    for (i = 0; i < NMAX; i++) {
      encoder.ref[i] = actual;
    }
    len = NMAX;
    newmode = HOLD_POSITION;
  } else if (ISCMD(cmd, DIS)) {
    // we are just disabling the motors
    NU32_WriteUART1("Motor disabled.\r\n");
    STOP();
    newmode = DO_NOTHING;
    // don't update length, so we can keep the old data around.
    len = encoder.len;
  } else if (ISCMD(cmd, TRAJ)) {
    // we are tracking a reference trajectory
    NU32_WriteUART1("Tracking trajectory.\r\n");
    GetEncoder(TRUE);
    len = encoder.len;
    newmode = TRACK_POSITION;
  } else {
    // huh?
    NU32_WriteUART1("???\r\n");
    return;
  }

 // reset arrays that store data.  ResetSignal caps len at NMAX, if too big.
 ResetSignal(&current, len);
 ResetSignal(&encoder, len);
 ResetSignal(&fast_current, 20 * len);
 // set new mode
 mode = newmode;
}

void SetCommand(char *cmd, char *data) {
  int tmp1, tmp2;
  if (ISCMD(cmd, DCMO)) {
    mode = DO_NOTHING; // tell ISRs to not control current
    // manually set PWM value and direction for DC motor
    sscanf(data, "%d %d", &tmp1, &tmp2);
    // check inputs are in valid ranges
    if ((tmp1 == 1 || tmp1 == 0) &&
            (tmp2 >= 0 && tmp2 <= 100)) {
      // set globals to reflect sent values
      dir = tmp1;
      duty1_percent = tmp2;
      PWM_Set_DC(duty1_percent); // set PWM
      // set direction
      if (dir) CW();
      else CCW();
    } else {
      NU32_WriteUART1("bad dcmo values\r\n");
      // stop motors:
      STOP();
    }
  } else if (ISCMD(cmd, ENC)) {
    // zero encoder counts
    sprintf(msg_out, "DC Motor -- zeroing encoder: %d\r\n", (int) GetEncoder(TRUE));
    NU32_WriteUART1(msg_out);
  } else if (ISCMD(cmd, CGAINS)) {
    // adjust current control gains
    sscanf(data, "%f %f", &Kp_curr, &Ki_curr);
  } else if (ISCMD(cmd, MGAINS)) {
    // adjust motion control gains
    sscanf(data, "%f %f", &Kp, &Ki);    
  } else {
    // unrecognized command
    NU32_WriteUART1("???\r\n");
  }
}


// This function gets called for the top-level command "get".  It is
// used for getting the status or value of many global variables.
void GetCommand(char *cmd) {
  if (ISCMD(cmd, DCMO)) {
    // get PWM values for DC motor
    sprintf(msg_out, "DC Motor -- dir: %d duty: %d (%d percent)\r\n",
            dir, duty1, duty1_percent);
    NU32_WriteUART1(msg_out);
  } else if (ISCMD(cmd, VER)) {
    // __DATE__ is a predefined C macro, others are __FILE__ and __LINE__.
    sprintf(msg_out, "Author: %s; Last Compiled: %s\r\n", AUTHOR, __DATE__);
    NU32_WriteUART1(msg_out);
  } else if (ISCMD(cmd, ENC)) {
    // return encoder reading
    sprintf(msg_out, "DC Motor -- encoder: %lld (counts)  %5.3f (degrees)\r\n",
	    (long long) encoder.actual,
	    encoder.actual*360.0/COUNTS_PER_REVOLUTION);
    NU32_WriteUART1(msg_out);
  } else if (ISCMD(cmd, CURR)) {
    // we just want to return the current value of the motor current
    unsigned int act_curr_adc = ADCManualRead(CURRENT_PIN);
    sprintf(msg_out, "Motor Current = %u counts (%f A)\r\n",
            act_curr_adc, ADC_TO_AMPS(VOLTS_PER_SAMPLE * act_curr_adc));
    NU32_WriteUART1(msg_out);
  } else if (ISCMD(cmd, CGAINS)) {
    // return the value of the current controller gains
    sprintf(msg_out, "Current controller gains: Kp = %f, Ki= %f\r\n",
            (double) Kp_curr, (double) Ki_curr);
    NU32_WriteUART1(msg_out);
  } else if (ISCMD(cmd, MGAINS)) {
    // return the value of the motion controller gains
    sprintf(msg_out, "Motion controller gains: Kp = %f, Ki= %f\r\n",
            (double) Kp, (double) Ki);
    NU32_WriteUART1(msg_out);
  } else {
    // unrecognized command
    NU32_WriteUART1("???\r\n");
  }
}

// This function gets called by the "load" top-level command. It is
// used for filling in the values of reference arrays.
void LoadCommand(char *traj) {
  int i = 0;
  char *tok;

  tok = strtok(traj, DELIM);  // strtok is for "tokenizing" strings in C
			      // i.e. for breaking up character arrays by
			      // some sort of delim (spaces, commas, etc.)
  while (tok != NULL && i < NMAX) {
    encoder.ref[i++] = atof(tok);  // atof() can convert a string to a float
    tok = strtok(NULL, DELIM);
  }

  encoder.len = i;
  sprintf(msg_out, "loaded %d samples into reference signal.\r\n", i);
  NU32_WriteUART1(msg_out);
}

// This function gets called for the "save" top-level command.  In here
// you can define different 
void SaveCommand(char *cmd) {
  NU32LED1 = 0;
  if (ISCMD(cmd, LOFREQ)) {
    SendArrays(4, encoder.len, "f f f f", encoder.ref, encoder.act,
            current.ref, current.act);
  } else if (ISCMD(cmd, HIFREQ)) {
    SendArrays(2, fast_current.len, "f f", fast_current.ref, fast_current.act);
  } else {
    NU32_WriteUART1("???\r\n");
  }
  /*
    else if (ISCMD(cmd, "test")) {
      // example code on the different types available
      char c[] = "0123456789";
      float f[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
      double d[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
      int i[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
      short s[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
      SendArrays(5, 10, "c f d i s", c, f, d, i, s);
    }
   */
  NU32LED1 = 1;
}


int SplitInput(char *str, char **cmd, char **data) {
  char *delim = NULL;
  *cmd = NULL;
  *data = NULL;

  // assume that the first space in str splits top-level from sub-level cmd.
  // This fails for commands like " top sub ...", but work for "top sub ...".
  delim = strchr(str, SPACE);
  *cmd = str;
  if (delim != NULL) {
    *data = delim + 1;
    delim[0] = '\0';
  }

  return (*cmd != NULL) && (*data != NULL);
}


void SendArrays(int rows, int cols, char *fmt, ...) {
 int i, j;
 va_list arrs;

 // this function is very fragile.  The fmt string must be a sequence of
 // valid characters separated by exactly one space.
 sprintf(msg_out, "%s %d ", SAVE, rows);
 NU32_WriteUART1(msg_out);
 for (i = 0; i < cols; i++) {
   va_start(arrs, fmt);
   for (j = 0; j < rows; j++) {
     // clear msg_out
     msg_out[0] = '\0';
     // what type is each array?
     if (fmt[2 * j] == 'c') {
       sprintf(msg_out, "%c ", (va_arg(arrs, char *))[i]);
     } else if (fmt[2 * j] == 's') {
       sprintf(msg_out, "%d ", (va_arg(arrs, short *))[i]);
     } else if (fmt[2 * j] == 'i') {
       sprintf(msg_out, "%d ", (va_arg(arrs, int *))[i]);
     } else if (fmt[2 * j] == 'f') {
       sprintf(msg_out, "%1.4f ", (double) (va_arg(arrs, float *))[i]);
     } else if (fmt[2 * j] == 'd') {
       sprintf(msg_out, "%1.4f ", (va_arg(arrs, double *))[i]);
     }
     NU32_WriteUART1(msg_out);
   }
   va_end(arrs);
 }

 // end of data, now send newline
 sprintf(msg_out, "\r\n");
 NU32_WriteUART1(msg_out);
}
