#include <plib.h>
#include "MyCommands.h"
#include "MyData.h"
#include "MyLibrary.h"
#include "NU32.h"
#include "math.h"


/********/
/* ISRs */
/********/

// ISR for the current control loop
void __ISR(_TIMER_4_VECTOR, IPL7) CurrentControlISR(void) {
  static float e = 0.0;
  float ref_curr, act_curr;
  float u = 0.0;

  // get the current value for the motor current, this must be done before
  // returning from the function.  It keeps the values up-to-date for get cmds
  ref_curr = current.desired;
  act_curr = ADC_TO_AMPS(VOLTS_PER_SAMPLE * ADCManualRead(CURRENT_PIN));
  UpdateSignal(&fast_current, ref_curr, act_curr);

  if (mode == DO_NOTHING) {
    IFS0bits.T4IF = 0;
    return;
  }

  // run the PI controller:
  e = ref_curr - act_curr;
  u = -(Kp_curr * e);
  // set direction
  if (u < 0)
    CCW();
  else if (u >= 0)
    CW();
  // handle max control effort:
  if (u < -(TMR2_PR + 1))
    u = (TMR2_PR + 1);
  else if (u > TMR2_PR + 1)
    u = TMR2_PR + 1;
  // set the new value for the motor control
  OC1RS = (unsigned) fabs(u);
  // clear the interrupt flag
  IFS0bits.T4IF = 0;
}

void __ISR(_TIMER_5_VECTOR, IPL5) MotionControlISR(void) {
  static int count;
  float desired_current;
  float actual_current;
  int index, n;
  static float e, u;

  count++;
  if (count%100 == 0)
    NU32LED2 = !NU32LED2;

  // obtain the current encoder reading, and update arrays:
  encoder.actual = (float) GetEncoder(FALSE);
  n = encoder.len - 1;
  index = encoder.index < n ? encoder.index : n;
  UpdateSignal(&encoder, encoder.ref[index], encoder.actual);
  
  if (mode == DO_NOTHING) {
    desired_current = 0;
  } else if (mode == TRACK_POSITION || mode == HOLD_POSITION) {
    // we are tracking a trajectory, let's run a motion control PI loop to
    // set the reference current
    e = encoder.desired - encoder.actual;
    u = (Kp*e)/MOTION_DIVISOR;
    if (u > UMAX) u = UMAX;
    else if (u < -UMAX) u = -UMAX;
    desired_current = u;    
      
    if(encoder.index >= encoder.len && mode != HOLD_POSITION) {
      // we are done with the trajectory
      NU32_WriteUART1("done tracking trajectory\r\n");
      mode = HOLD_POSITION;
    }
  } else if (mode == TUNE_CURRENT) {
    desired_current = !(current.index % TOGGLE_FREQ) ?
      current.desired : -current.desired;
    if (current.index >= current.len) {
      // we are done with the trajectory
      mode = DO_NOTHING;
      NU32_WriteUART1("done current tuning\r\n");
    }
  }

  // update low speed current data and set desired current
  actual_current = ADC_TO_AMPS(VOLTS_PER_SAMPLE *
			       ADCManualRead(CURRENT_PIN));
  // every call to UpdateSignal increments the index.
  UpdateSignal(&current, desired_current, actual_current);
  // clear the interrupt flag
  IFS0bits.T5IF = 0;
}


/*****************/
/* MAIN FUNCTION */
/*****************/
int main() {
  char *top, *sub;
  int i=0;
  
  STOP(); // disable motors
  mode = DO_NOTHING;
  // set global defaults
  SetGlobalDefaults();
  // initialize peripherals 
  InitCurrentControl(TMR4_PS_VAL, TMR4_PR);
  StartPWM(TMR2_PS_VAL, TMR2_PR);
  InitEncoderSPI();
  InitMotionControl(TMR5_PS_VAL, TMR5_PR);
  // call encoder function to get it properly setup to handle rollover
  for (i=0; i<2; i++)
    encoder.actual = (float) GetEncoder(TRUE);

  // nu32 initialization:
  NU32_Startup();
  NU32_Initialize();

  sprintf(msg_out, "Enter a command.\r\n");
  NU32_WriteUART1(msg_out);

  while (1) {
    NU32_ReadUART1(msg_in, IN_MAX);
    if (strlen(msg_in) == 0) {
      continue; // empty string
    }

    if (SplitInput(msg_in, &top, &sub) == FALSE) {
      continue; // invalid string format
    }

    // process top-level command
    if (ISCMD(top, LOAD)) {
      LoadCommand(sub);
    } else if (ISCMD(top, SAVE)) {
      SaveCommand(sub);
    } else if (ISCMD(top, SET)) {
      SplitInput(sub, &top, &sub);
      SetCommand(top, sub);
    } else if (ISCMD(top, GET)) {
      GetCommand(sub);
    } else if (ISCMD(top, EXEC)) {
      ExecCommand(sub);
    } else {
      NU32_WriteUART1("???\r\n");
    }
  }
  return 0;
}
